
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                            %
%                       ForWaRD Algorithm Implementation                     %
%                              (1-D signal deblurring)                       %
%                                                                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% This code specifies the experimental setup required to demonstrate the
% performance of Fourier-Wavelet Regularized Deconvolution (ForWaRD)
% algorithm proposed by Ramesh Neelamani, Hyeokho Choi and Richard Baraniuk
% from Rice University.
% 
% Reference: R. Neelamani, H. Choi, and R. G. Baraniuk, ``ForWaRD: 
% Fourier-Wavelet Regularized Deconvolution for Ill-Conditioned Systems,''
% IEEE Transactions on Signal Processing, p 418-433, February 2004. 
% Check http://www.dsp.rice.edu/publications/ to
% download the reference.
%
% Your comments would be highly appreciated. Please send them to 
% neelsh@ece.edu
%
% This program uses some routines from the  Rice Wavelet toolbox.
%

clear all
addpath 'routines1D'

% number of samples 
N = 2^12;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input signal creation.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

x = [makesig('HeaviSine',N/2) makesig('Blocks',N/2)]; % See makesig.m for other choices

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Convolution operator creation.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (1) Box car blur (default 10-point)
% (2) Dilation-homogeneous operator (polynomial frequency decay; default
%     decay = 1)
% (3) 0.5*exp(-|x|)
Blur = 1;
if Blur == 1
  BoxCarBlurSize = 10;
  h = zeros(1,N);
  h(1:BoxCarBlurSize) = 1/BoxCarBlurSize;
  H = fft(h);
  nu = 3/2; % this influences the thresholds = sqrt((4*nu+2)*log(N))
elseif Blur == 2
  FreqDecay = 0.75;
  % dilation-homogeneous operator (suited for WVD)
  H = 200./([1:(N/2+1) (N/2):-1:2].^FreqDecay);
  h = real(ifft(H));
  nu = FreqDecay; % this influences the thresholds = sqrt((4*nu+2)*log(N))
else  
  lambda = 0.25;
  h = (lambda/(2))*exp(lambda*-abs([1:(N/2+1) (N/2):-1:2] -1));
  H = fft(h);
  nu = 1; % this influences the thresholds = sqrt((4*nu+2)*log(N))
end

   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Noise creation.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% What blurred-signal-to-noise ratio (in dB) would you like?
% If y = x*h + n is the decon model, BSNR = 10log10(|norm(x*h)/norm(n)|^2)
% default is 20 dB (* denotes convolution)

BSNR = 20;
tempvar = real(ifft(fft(x) .* fft(h)));
OrgSigma = sqrt(norm(tempvar(:)-mean(tempvar(:)),2)^2 /(N*10^(BSNR/10)));
n = OrgSigma*randn(1,N);
clear tempvar

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Observation creation.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
n = n(:); x = x(:); h = h(:); H = H(:);
y = real(ifft(fft(x) .* H)) + n ;
Y = fft(y);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Wavelet Basis
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Two wavelet basis are required for implementing the wavelet-domain wiener
% filter (WDWF) used to perform the estimation. See daubcqf.m to set the
% type wavelet filter.

WaveletFiltType1 = daubcqf(6,'min'); 
WaveletFiltType2 = daubcqf(4,'min'); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Wavelet Decomposition levels
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\
% How many levels of decomposition do you want?
% The maximum possible is log2(N) (default).
DecompLevels = log2(N);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thresholding parameter
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% What threshold should be used in for hard thresholding (1st stage in
% WDWF)?  default threshold is sqrt(2*log(N))*sigma, sigma is standard
% deviation of noise.
% suggested (aggressive threshold) = sqrt((4*nu+2)*log(N))
% using milder threshold = sqrt((1.1*nu+2)*log(N));
ThreshFactor = sqrt((1.1*nu+2)*log(N));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Estimating Noise variance and regularization parameter 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Do you want to estimate the noise variance of the input
% signal from the observation?
% Choices are Variance = 'Estimate'
%             Variance = 'Original'

Variance = 'Estimate';


% Do you want the regularization parameter to be automatically estimated?
% Choices are RegSearch = 'Y' (recommended, but about 5 times slower) 
%             RegSearch = 'N' (default)
RegSearch = 'Y';

% If RegSearch = 'N', then you need to specify a regularization parameter.
% The choice is normalized with respect to the noise variance and average 
% signal power. Choice of RegParam is ignored if RegSearch = 'Y'.

RegParam = 1;

% Do you want to use the Original signal to cross check the estimation of  
% the SNR and choice of regularization parameter? 
% 
OrgSigAvailable = 1;

% This sets up the reg parameters and noise variances 

RegDebug = 0; % this is for debug purposes.
ForWaRDSetup1D

% This searches the regularization parameter.
RegParamSetup1D

% We are now in a position to run the ForWaRD algorithm.
RunForWaRD1D

% Running the Wiener filtering algorithm.
Iterations = 10; % Number of Iterations to estimation the power
                % spectral density
RunWiener1D

% Running the WVD (Donoho) algorithm. This is equivalent to running 
% ForWaRD with zero (in practice, regparam = eps) regularization parameter 
% This is not designed to handle Boxcar-type (Blur = 1) convolution
% operators and would yield very poor results. It is suited for
% dilation-homogeneous operators (Blur = 2)
RunWVD1D

% We are now in a position to display the results
DispResults1D




