function [xward,xwiener,xwvd] = ForWaRDFunction(y,h,ThreshFactor,DecompLevels,RegParam,x,figplot);
% Usage: [xward,xwiener,xwvd] = ForWaRDFunction(y,h,ThreshFactor,DecompLevels,RegParam,x,figplot);
%
% Outputs:
% xward: forward estimate
% xwiener: wiener estimate (not very robust)
% xwvd: WVD estimate (obtained by setting regularization param = 0)
%
% Required inputs:
% y = observation
% h = blur
%
% Optional inputs:
% ThreshFactor = sqrt((1.1*nu+2)*log(N)), with nu the approximate
%                Fourier decay of the convolution operator. Used in 
%                wavelet thresholding. This ThreshFactor can be 
%                adapted appropriately.
% DecompLevels = Levels of wavelet decomposition
% RegParam = RegParam used during operator inversion. Set to -1 to
%            automatically perform search
% x = Original signal. Used only to computed SNRs and evaluate choice of 
%     regularization parameter
% figplot = set to 1 for plotting results
%
% NOTE: Needs functions in routines1D directory. Add to path before 
%       starting.
%
% Reference: R. Neelamani, H. Choi, and R. G. Baraniuk, ``ForWaRD: 
% Fourier-Wavelet Regularized Deconvolution for Ill-Conditioned Systems,''
% IEEE Transactions on Signal Processing, p 418-433, February 2004. 
% Check http://www.dsp.rice.edu/publications/ to
% download the reference.
%
% Your comments would be highly appreciated. Please send them to 
% neelsh@ece.edu
%
% This program uses some routines from the  Rice Wavelet toolbox.
%


% number of samples 
y = y(:);
N = length(y);
H = fft(h);
h = h(:); H = H(:);
Y = fft(y);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Wavelet Basis
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
WaveletFiltType1 = daubcqf(10,'min'); 
WaveletFiltType2 = daubcqf(6,'min'); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Thresholding parameter
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% What threshold should be used in for hard thresholding (1st stage in
% WDWF)?  default threshold is sqrt(2*log(N))*sigma, sigma is standard
% deviation of noise.
if nargin < 3
  nu = 1;
  ThreshFactor = sqrt((1.1*nu+2)*log(N));
elseif ThreshFactor < 0 
  nu = 1;
  ThreshFactor = sqrt((1.1*nu+2)*log(N));
end
%disp(strcat('ThreshFactor=',num2str(ThreshFactor)));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Wavelet Decomposition levels
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\
if nargin < 4
  DecompLevels = log2(N);
elseif DecompLevels <=0 
  DecompLevels = log2(N);
end



Variance = 'Estimate';


% Do you want the regularization parameter to be automatically estimated?
% Choices are RegSearch = 'Y' (recommended, but about 5 times slower) 
%             RegSearch = 'N' (default)
if nargin < 5
  RegSearch = 'Y';
  RegParam = 1;
else
  if RegParam < 0
    RegSearch = 'Y';
    RegParam = 1;
  else
    RegSearch = 'N';
  end
end


% Do you want to use the Original signal to cross check the estimation of  
% the SNR and choice of regularization parameter? 
% 
if nargin < 6
  OrgSigAvailable = 0;
else
  if length(x) < length(y)
    OrgSigAvailable = 0;
  else
    OrgSigAvailable = 1;
    x = x(:); 
  end
end


if nargin < 7
  figplot = 1;
end


% This sets up the reg parameters and noise variances 

RegDebug = 0; % this is for debug purposes.
ForWaRDSetup1D

% This searches the regularization parameter.
RegParamSetup1D

% We are now in a position to run the ForWaRD algorithm.
RunForWaRD1D

% Running the Wiener filtering algorithm.
Iterations = 10; % Number of Iterations to estimation the power
                % spectral density
RunWiener1D

% Running the WVD (Donoho) algorithm. This is equivalent to running 
% ForWaRD with zero (in practice, regparam = eps) regularization parameter 
% This is not designed to handle Boxcar-type (Blur = 1) convolution
% operators and would yield very poor results. It is suited for
% dilation-homogeneous operators (Blur = 2)
RunWVD1D


% We are now in a position to display the results
if figplot 
  DispResults1D
end



